/*
+--------------------------------------------------------------------
| TERMS OF USE: MIT License
+--------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+--------------------------------------------------------------------
*/

////////////////////////////////////////////////////////////////////////
// compile: fastspin -2b pingpong.c graphictools.c
// Thanks to rogloh for the videodriver and graphicroutines
// and of course to ersmith for the language
////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include <propeller2.h>
#define P2_TARGET_MHZ 160
#include "sys/p2es_clock.h"

#include "graphictools.h"

#define BAUD 230400

void ball(int x0,int y0, int radius, int color);
void paddle (int ym, int color);
void paddle2 (int ym, int color);

void main()
{
     int i;
	
    _clkset(_SETFREQ, _CLOCKFREQ);
    Setup_Video();
    _setbaud(BAUD);
    
    int xx = 20;
    int yy = 20;
    int dx = 5;
    int dy = 5;
    int ym = 230;
    int ym2 = 230;
    float r;
    
    clearscreen(BLUE);
    for (;;) // mainlooop
    {
		ball(xx, yy, 5, WHITE);
		paddle(ym, WHITE);
		paddle2(ym2,WHITE);
        vid.waitforblanking(&display1);
        ball(xx,yy, 5, BLUE);
		paddle(ym, BLUE);
		paddle2(ym2, BLUE);
		
		xx += dx;
		yy += dy;
		
		// check screen limits
		if(xx < 20) dx = -dx;
		if(xx > 620) dx = -dx;
		if(yy < 20) dy = -dy;
		if(yy > 460) dy = -dy;
		
		// check relation from ball to paddle
		if ( ((xx+5) == 610) && 
		     (((yy) < (ym-10)) || ((yy) < (ym+10)))  
		   )
			dx = -dx;
		
		if ( ((xx-5) == 30) && 
		     (((yy) < (ym2-10)) || ((yy) < (ym2+10)))  
		   )
			dx = -dx;			

		// Find intersection ball - paddle
		// Solve system of equations:
		//    Paddle = Ball
		// x: 610    = xx + r * dx
		// y:  ym    = yy + r * dy	
		
		//Player1
		if(dx > 0)
		{
			r = (float)((610 - xx) / dx);
			ym = yy + (int)r * dy;
		}
		
		//Player2
		if(dx < 0)
		{
			r = (float)((30 - xx) / dx);
			ym2 = yy + (int)r * dy;
		}		
		
		// check working space for paddle
		if(ym < 10 || ym > 470)
		  ym = 230;
		
		if(ym2 < 10 || ym2 > 470)
		  ym2 = 230;		  
	
	}//end mainloop

}//end main


void ball(int x0,int y0, int radius, int color)
{
	fillCircle(&frameBuffer, SCRN_WIDTH*8/8, x0, y0, radius, color, 8); // 8bpp
}

void paddle (int ym, int color)
{
	int yo = ym - 10;
	int yu = ym + 10;
	line(610,yu,610,yo,color);
}

void paddle2 (int ym, int color)
{
	int yo = ym - 10;
	int yu = ym + 10;
	line(30,yu,30,yo,color);
}
